package com.koobietech.eas.mbg.mapper;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.koobietech.eas.mbg.model.EasSysRoleusers;
import com.koobietech.eas.mbg.model.EasSysRoleusersExample;
import io.swagger.v3.oas.annotations.media.Schema;
import java.util.List;
import org.apache.ibatis.annotations.Param;

public interface EasSysRoleusersMapper {
    long countByExample(EasSysRoleusersExample example);

    int deleteByExample(EasSysRoleusersExample example);

    int insert(EasSysRoleusers record);

    int insertSelective(EasSysRoleusers record);

    List<EasSysRoleusers> selectByExample(EasSysRoleusersExample example);

    int updateByExampleSelective(@Param("record") EasSysRoleusers record, @Param("example") EasSysRoleusersExample example);

    int updateByExample(@Param("record") EasSysRoleusers record, @Param("example") EasSysRoleusersExample example);
}